;######################################################################## 
;######################################################################## 
;######################################################################## 
TWI_INITIALISIERUNG:
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|1<<TWIE
	out TWCR,temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	ldi temp,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
	out TWAR,temp

	sbi PORTC,4						; PULL-UP ein							
	sbi PORTC,5						; PULL-UP ein							
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_ERROR:
	ldi temp1,30
TWI_ERROR_S:
	ldi temp, 1<<TWINT|1<<TWSTO|1<<TWEN|1<<TWIE  ; STOP  => wichtig
	out TWCR,temp
	dec temp1
	brne TWI_ERROR_S
	; FEHLERPRFUNG
	in temp, TWSR
	andi temp,0b11111000
	cpi  temp,0b11111000
	brne TWI_ERROR_SCHWER	
	ret
;--------------------------------------------------------------
TWI_ERROR_SCHWER:
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR	; ROT  an					
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; TWI INTERRUPPT-Routine														
;																				
TWSI:
	push TEMP		    		; Sichern von "TEMP" im Stack			
	in	 TEMP,SREG				; Einlesen des SREG 					
	push TEMP	    			; Schreiben von  SREG  im Stack (KOPIE)	
	push TEMP1
	push TEMP2
	push YL
	push YH
;-------------------------------------------------------------------------------
	in temp, TWSR			; TWI- Status-Register abfragen							
	andi temp, 0b11111000	; (0xF8)Bit0...2 ausblenden (Prescaler und Reserve-Bit)	
;-------------------------------------------------------------------------------
TWSI_TWSR_AUSWERTEN:
	;	vergleich		
	cpi temp,(0x60)
	breq TWSI_TW_SR_SLA_ACK
	cpi temp,(0x68)
	breq TWSI_TW_SR_ARB_LOST_SLA_ACK
	cpi temp,(0x70)
	breq TWSI_TW_SR_GCALL_ACK
	cpi temp,(0x78)
	breq TWSI_TW_SR_ARB_LOST_GCALL_ACK
	cpi temp,(0x80)
	breq TWSI_TW_SR_DATA_ACK      	
	cpi temp,(0x88)
	breq TWSI_TW_SR_DATA_NACK
	cpi temp,(0x90)
	breq TWSI_TW_SR_GCALL_DATA_ACK   
	cpi temp,(0x98)
	breq TWSI_TW_SR_GCALL_DATA_NACK
	cpi temp,(0xA0)
	breq TWSI_TW_SR_STOP        	
; 	
	cpi temp,(0xA8)
	breq TWSI_TW_ST_SLA_ACK	
	cpi temp,(0xB0)
	breq TWSI_TW_ST_ARB_LOST_SLA_ACK
	cpi temp,(0xB8)
	breq TWSI_TW_ST_DATA_ACK
	cpi temp,(0xC0)
	breq TWSI_TW_ST_DATA_NACK
	cpi temp,(0xC8)
	breq TWSI_TW_ST_LAST_DATA 
	cpi temp,(0xF8)
	breq TWSI_TW_NO_INFO
	cpi temp,(0x00)			;(Error)
	breq TWSI_TW_BUS_ERROR 
	rjmp TWSI_ANTWORT		; nicht behandelte Status-Zustnde	
;-----------------------------------------------------------------------------------------------------------------------
; SLAVE RECEIVER						
TWSI_TW_SR_SLA_ACK:						; x60 ("SLA+W" wurde empfangen) 				
	; START				
	rjmp TWSI_ANTWORT
TWSI_TW_SR_ARB_LOST_SLA_ACK:			; x68 (Arbitration verloren)					
	; NIX tun			
	rjmp TWSI_ANTWORT
TWSI_TW_SR_GCALL_ACK:					; x70 ("General Call" wurde empfangen)			
	; START "General Call"		
	rjmp TWSI_ANTWORT
TWSI_TW_SR_ARB_LOST_GCALL_ACK:			; x78 (Arbitration verloren)					
	; NIX tun			
	rjmp TWSI_ANTWORT
TWSI_TW_SR_DATA_ACK:					; x80 ("SLA+W"-Datenbyte wurden empfangen) 		
	; DATA								
	in temp1, TWDR		
	rcall BUFFER_WRITE_TEMP1
	rjmp TWSI_ANTWORT
TWSI_TW_SR_DATA_NACK:      				; x88 (die vorher adressierten (SLA+W) Daten wurden empfangen; ACK wurde nicht zurckgesedet
	; NIX tun			
	rjmp TWSI_ANTWORT
TWSI_TW_SR_GCALL_DATA_ACK:				; x90 ("General Call"-Datenbyte wurden empfangen)	
	; DATA								
	in temp1, TWDR		
	rcall BUFFER_WRITE_TEMP1		
	rjmp TWSI_ANTWORT
TWSI_TW_SR_GCALL_DATA_NACK:				; x98 (die vorher adressierten (General call) Daten wurden empfangen;  ACK wurde nicht zurckgesendet	
	; NIX tun			
	rjmp TWSI_ANTWORT
TWSI_TW_SR_STOP:						; xA0 (STOP oder Repead-START wurde empfangen)
	; STOP				
	rjmp TWSI_ANTWORT
;-----------------------------------------------------------------------------------------------------------------------
;	SLAVE TRANSMITTER																									
TWSI_TW_ST_SLA_ACK:						; xA8 (SLA+R wurde empfangen) + ACK 
	; DATA			
	ldi TEMP,0b00000001 ; TEST
	OUT TWDR,TEMP 						; DATA zur Verfgung stellen
	rjmp TWSI_ANTWORT
TWSI_TW_ST_ARB_LOST_SLA_ACK:			; xB0 (Arbitration verlor SLA+R/W als MASTER, ACK wurde zurckgesendet)	
	rjmp TWSI_ANTWORT
TWSI_TW_ST_DATA_ACK:					; xB8 (DATEN wurden bertragen) + ACK 	
	; DATA			
	ldi TEMP,0b00000010 ; TEST
	OUT TWDR,TEMP 						; DATA zur Verfgung stellen
	rjmp TWSI_ANTWORT
TWSI_TW_ST_DATA_NACK:					; xC0 (TWDR ist bertragen worden;  ACK ist NICHT empfangen worden
	; DATA			
	ldi TEMP,0b00000100 ; TEST
	OUT TWDR,TEMP 						; DATA zur Verfgung stellen
	rjmp TWSI_ANTWORT
TWSI_TW_ST_LAST_DATA:					; xC8 (Letztes Datenbyte in TWDR ist bertragen worden (TWEA = "0");  Ack ist empfangen worden
	; DATA			
	ldi TEMP,0b00001000 ; TEST
	OUT TWDR,TEMP 						; DATA zur Verfgung stellen
	rjmp TWSI_ANTWORT
TWSI_TW_NO_INFO:						; xF8 (Keine relevanten Zustandinformationen vorhanden;  TWINT = "0" keine TWDR-Ttigkeit 
	rjmp TWSI_ANTWORT
; ERROR									
TWSI_TW_BUS_ERROR:						; x00 (Bus-Error wegen ungltige STARToder STOP BEDIgung; keine TWDR-Ttigkeit 
	rcall TWI_ERROR	
	rjmp TWSI_ANTWORT
;--------------------------------------
TWSI_ANTWORT:	
	ldi temp, 1<<TWINT|1<<TWEA|1<<TWEN|1<<TWIE; ANTWORTEN							
	out TWCR,temp
;--------------------------------------
	pop YH
	pop YL
	pop temp2	
	pop temp1
	pop	TEMP				; LESEN von SREG vom STACK (KOPIE)	
	out	SREG,TEMP			; Wiederherstellen von SREG			
	pop	TEMP				; Wiederherstellen von TEMP			
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################

